# -*- coding: utf-8 -*-

# Copyright (c) 2005 - 2006 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a dialog to configure the PyLint process
"""

import os
import sys
import copy

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from KdeQt import KQFileDialog, KQMessageBox
from KdeQt.KQApplication import e4App

from Ui_PyLintConfigDialog import Ui_PyLintConfigDialog

import Utilities

class PyLintConfigDialog(QDialog, Ui_PyLintConfigDialog):
    """
    Class implementing a dialog to configure the PyLint process
    """
    def __init__(self, ppath, parms = None, parent = None):
        """
        Constructor
        
        @param ppath project path (string or QString)
            Used to set the default path for the rcfile selection dialog
        @param parms parameters to set in the dialog
        @param parent reference to the parent widget (QWidget)
        """
        QDialog.__init__(self, parent)
        self.setupUi(self)
        
        self.pylintProc = None
        self.lint = 'pylint'
        if sys.platform == "win32":
            self.lint = os.path.join(sys.exec_prefix, "Scripts", self.lint + '.bat')
        
        self.__initializeDefaults()
        
        # get a copy of the defaults to store the user settings
        self.parameters = copy.deepcopy(self.defaults)
        
        # combine it with the values of parms
        if parms is not None:
            for key, value in parms.items():
                self.parameters[key] = parms[key]
        
        # initialize general tab
        self.configfileEdit.setText(self.parameters['configFile'])
        self.txtOutputButton.setChecked(self.parameters['txtReport'])
        self.htmlOutputButton.setChecked(self.parameters['htmlReport'])
        self.dialogOutputButton.setChecked(self.parameters['dialogReport'])
        self.reportfileEdit.setText(self.parameters['reportFile'])
        
        # initialize checkers tab
        self.basicCheckBox.setChecked(self.parameters['enableBasic'])
        self.classesCheckBox.setChecked(self.parameters['enableClasses'])
        self.designCheckBox.setChecked(self.parameters['enableDesign'])
        self.exceptionsCheckBox.setChecked(self.parameters['enableExceptions'])
        self.formatCheckBox.setChecked(self.parameters['enableFormat'])
        self.importsCheckBox.setChecked(self.parameters['enableImports'])
        self.metricsCheckBox.setChecked(self.parameters['enableMetrics'])
        self.miscellaneousCheckBox.setChecked(self.parameters['enableMiscellaneous'])
        self.similaritiesCheckBox.setChecked(self.parameters['enableSimilarities'])
        self.variablesCheckBox.setChecked(self.parameters['enableVariables'])
        
        self.ppath = ppath

    def __initializeDefaults(self):
        """
        Private method to set the default values. 
        
        These are needed later on to generate the commandline
        parameters.
        """
        self.defaults = {
            # general options
            'configFile' : '',
            'reportFile' : '',
            'txtReport' : False,
            'htmlReport' : True,
            'dialogReport' : False,
            
            # enabled checkers
            'enableBasic'         : False,
            'enableClasses'       : False,
            'enableDesign'        : False,
            'enableExceptions'    : False,
            'enableFormat'        : False,
            'enableImports'       : False,
            'enableMetrics'       : True,
            'enableMiscellaneous' : False,
            'enableSimilarities'  : False,
            'enableVariables'     : False,
        }

    def generateParameters(self):
        """
        Public method that generates the commandline parameters.
        
        It generates a QStringList to be used
        to set the QProcess arguments for the pylint call and
        a list containing the non default parameters. The second
        list can be passed back upon object generation to overwrite
        the default settings.
        
        <b>Note</b>: The arguments list contains the name of the pylint executable as
        the first parameter.
        
        @return a tuple of the commandline parameters and non default parameters
            (QStringList, dictionary)
        """
        parms = {}
        args = QStringList()
        
        # 1. the program name
        if sys.platform == "win32":
            args.append(sys.executable)
        args.append(self.lint)
        
        # 2. the commandline options
        # 2.1 general options
        if self.parameters['configFile'] != self.defaults['configFile']:
            parms['configFile'] = self.parameters['configFile']
##            args.append('--rcfile=%s' % self.parameters['configFile'])
            args.append('--rcfile')
            args.append(self.parameters['configFile'])
        parms['txtReport'] = self.parameters['txtReport']
        parms['htmlReport'] = self.parameters['htmlReport']
        parms['dialogReport'] = self.parameters['dialogReport']
        if self.parameters['htmlReport']:
            args.append('--html=y')
        elif self.parameters['dialogReport']:
            args.append('--parseable=y')
            args.append('--report=n')
        if self.parameters['reportFile'] != self.defaults['reportFile']:
            parms['reportFile'] = self.parameters['reportFile']
        
        # 2.2 checkers options
        parms['enableBasic'] = self.parameters['enableBasic']
        parms['enableClasses'] = self.parameters['enableClasses']
        parms['enableDesign'] = self.parameters['enableDesign']
        parms['enableExceptions'] = self.parameters['enableExceptions']
        parms['enableFormat'] = self.parameters['enableFormat']
        parms['enableImports'] = self.parameters['enableImports']
        parms['enableMetrics'] = self.parameters['enableMetrics']
        parms['enableMiscellaneous'] = self.parameters['enableMiscellaneous']
        parms['enableSimilarities'] = self.parameters['enableSimilarities']
        parms['enableVariables'] = self.parameters['enableVariables']
        
        args.append('--disable-all')
        if self.parameters['enableBasic']:
            args.append('--enable-basic=y')
        if self.parameters['enableClasses']:
            args.append('--enable-classes=y')
        if self.parameters['enableDesign']:
            args.append('--enable-design=y')
        if self.parameters['enableExceptions']:
            args.append('--enable-exceptions=y')
        if self.parameters['enableFormat']:
            args.append('--enable-format=y')
        if self.parameters['enableImports']:
            args.append('--enable-imports=y')
        if self.parameters['enableMetrics']:
            args.append('--enable-metrics=y')
        if self.parameters['enableMiscellaneous']:
            args.append('--enable-miscellaneous=y')
        if self.parameters['enableSimilarities']:
            args.append('--enable-similarities=y')
        if self.parameters['enableVariables']:
            args.append('--enable-variables=y')
        
        return (args, parms)

    @pyqtSignature("")
    def on_configfileButton_clicked(self):
        """
        Private slot to select the configuration file.
        
        It displays a file selection dialog to select the configuration file.
        """
        startWith = self.configfileEdit.text()
        if startWith.isEmpty():
            startWith = self.ppath
        config = KQFileDialog.getOpenFileName(\
            self,
            self.trUtf8("Select configuration file"),
            startWith,
            self.trUtf8("Configuration Files (*.cfg *.cnf *.rc);;All Files (*)"))
        if not config.isEmpty():
            self.configfileEdit.setText(Utilities.toNativeSeparators(config))
    
    @pyqtSignature("")
    def on_reportfileButton_clicked(self):
        """
        Private slot to select the report file.
        
        It displays a file selection dialog to select the report file.
        """
        report = KQFileDialog.getSaveFileName(\
            self,
            self.trUtf8("Select report file"),
            self.reportfileEdit.text(),
            QString(),
            None,
            QFileDialog.Options(QFileDialog.DontConfirmOverwrite))
        
        if not report.isEmpty():
            self.reportfileEdit.setText(Utilities.toNativeSeparators(report))
    
    def on_configfileEdit_textChanged(self, txt):
        """
        Private slot to handle the TextChanged signal of configfileEdit.
        
        @param txt contents of configfileEdit (QString)
        """
        self.okButton.setEnabled(not txt.isEmpty())
        
    def accept(self):
        """
        Protected slot called by the Ok button. 
        
        It saves the values in the parameters dictionary.
        """
        # get data of general tab
        self.parameters['configFile'] = unicode(self.configfileEdit.text())
        self.parameters['txtReport'] = self.txtOutputButton.isChecked()
        self.parameters['htmlReport'] = self.htmlOutputButton.isChecked()
        self.parameters['dialogReport'] = self.dialogOutputButton.isChecked()
        self.parameters['reportFile'] = unicode(self.reportfileEdit.text())
        
        # get data of checkers tab
        self.parameters['enableBasic'] = self.basicCheckBox.isChecked()
        self.parameters['enableClasses'] = self.classesCheckBox.isChecked()
        self.parameters['enableDesign'] = self.designCheckBox.isChecked()
        self.parameters['enableExceptions'] = self.exceptionsCheckBox.isChecked()
        self.parameters['enableFormat'] = self.formatCheckBox.isChecked()
        self.parameters['enableImports'] = self.importsCheckBox.isChecked()
        self.parameters['enableMetrics'] = self.metricsCheckBox.isChecked()
        self.parameters['enableMiscellaneous'] = self.miscellaneousCheckBox.isChecked()
        self.parameters['enableSimilarities'] = self.similaritiesCheckBox.isChecked()
        self.parameters['enableVariables'] = self.variablesCheckBox.isChecked()
        
        # call the accept slot of the base class
        QDialog.accept(self)

    ############################################################################
    ## Methods below are needed to generate a configuration file template
    ############################################################################

    @pyqtSignature("")
    def on_configButton_clicked(self):
        """
        Public slot to handle the generation of a sample configuration.
        """
        self.buf = QString("")
        self.pylintProc = QProcess()
        args = QStringList()
        
        if sys.platform == "win32":
            program = sys.executable
            args.append(self.lint)
        else:
            program = self.lint
        args.append('--generate-rcfile')
        
        self.connect(self.pylintProc, SIGNAL('readyReadStandardOutput()'),
            self.__readStdout)
        self.connect(self.pylintProc, SIGNAL('readyReadStandardError()'),
            self.__readStderr)
        self.connect(self.pylintProc, SIGNAL('finished(int, QProcess::ExitStatus)'),
            self.__createConfigDone)
        
        self.pylintProc.start(program, args)
        procStarted = self.pylintProc.waitForStarted()
        if procStarted:
            e4App().getObject("ViewManager").enableEditorsCheckFocusIn(False)
        else:
            KQMessageBox.critical(self,
                self.trUtf8('Process Generation Error'),
                self.trUtf8(
                    'Could not start %1.<br>'
                    'Ensure that it is in the search path.'
                ).arg(self.lint))
    
    def __createConfigDone(self, exitCode, exitStatus):
        """
        Private slot to handle the the finished signal of the pylint process.
        
        @param exitCode exit code of the process (integer)
        @param exitStatus exit status of the process (QProcess.ExitStatus)
        """
        vm = e4App().getObject("ViewManager")
        vm.enableEditorsCheckFocusIn(True)
        if exitStatus == QProcess.NormalExit and exitCode == 0:
            vm.newEditor()
            aw = vm.activeWindow()
            aw.insertAt(self.buf, 0, 0)
            aw.setLanguage('dummy.rc')
        self.reject()
    
    def __readStdout(self):
        """
        Private slot to handle the readyReadStandardOutput signal of the pylint process.
        """
        if self.pylintProc is None:
            return
        self.pylintProc.setReadChannel(QProcess.StandardOutput)
        
        while self.pylintProc and self.pylintProc.canReadLine():
            self.buf.append(self.pylintProc.readLine())
            self.buf.append(os.linesep)
        
    def __readStderr(self):
        """
        Private slot to handle the readyReadStandardError signal of the pylint process.
        """
        if self.pylintProc is None:
            return
        self.pylintProc.setReadChannel(QProcess.StandardError)
        while self.pylintProc and self.pylintProc.canReadLine():
            s = QString('pylint: ')
            s.append(self.pylintProc.readLine())
            e4App().getObject("UserInterface").emit(SIGNAL('appendStderr'), s)
        
