# -*- coding: utf-8 -*-

# Copyright (c) 2006 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing the base class of all canvas items.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *

class E4CanvasItem(object):
    """
    Class implementing the base class of all canvas items.
    """
    def __init__(self, canvas, origin = QPointF(0.0, 0.0)):
        """
        Constructor
        
        @param canvas reference to the canvas object (E4Canvas)
        @param origin origin of the item (QPointF)
        """
        self._canvas = canvas
        self._x = origin.x()
        self._y = origin.y()
        self._z = 0.0
        self._selected = False
        self._visible = False
        self._pen = QPen()
        self._brush = QBrush()
        
        self._boundingRect = QRectF(self._x, self._y, 0.0, 0.0)
        
        self._path = QPainterPath()
        
        self._canvas.addItem(self)
        self._adaptCanvasSize()
        
    def getCanvas(self):
        """
        Public method to get a reference to the canvas object.
        
        @return reference to the canvas object (E4Canvas)
        """
        return self._canvas
        
    def setCanvas(self, canvas):
        """
        Public method to set the items canvas.
        
        @param canvas reference to the canvas object (E4Canvas)
        """
        if canvas != self._canvas:
            self._canvas = canvas
            self._selected = False
            self._canvas.addItem(self)
            self._adaptCanvasSize()
        
    def setX(self, x):
        """
        Public method to set the x-coordinate of the origin.
        
        @param x x-coordinate (float)
        """
        self._x = x
        
    def setY(self, y):
        """
        Public method to set the x-coordinate of the origin.
        
        @param y y-coordinate (float)
        """
        self._y = y
        
    def setZ(self, z):
        """
        Public method to set the z-coordinate of the item.
        
        @param z z-coordinate (float)
        """
        self._z = z
        
    def x(self):
        """
        Public method to get the x-coordinate.
        
        @return x-coordinate (float)
        """
        return self._x
        
    def y(self):
        """
        Public method to get the y-coordinate.
        
        @return y-coordinate (float)
        """
        return self._y
        
    def z(self):
        """
        Public method to get the z-coordinate.
        
        @return z-coordinate (float)
        """
        return self._z
        
    def getOrigin(self):
        """
        Public method to get the origin of the item.
        
        @return origin of the item (QPointF)
        """
        return QPointF(self._x, self._y)
        
    def getPen(self):
        """
        Public method to get the current pen.
        
        @return reference to the current pen (QPen)
        """
        return self._pen
        
    def getBrush(self):
        """
        Public method to get the current brush.
        
        @return reference to the current brush (QBrush)
        """
        return self._brush
        
    def setVisible(self, visible):
        """
        Public method to set the items visibility.
        
        @param visible visibility (boolean)
        """
        self._visible = visible
        
    def isVisible(self):
        """
        Public method to check the items visibility.
        
        @return flag indicating the visibility (boolean)
        """
        return self._visible
        
    def hide(self):
        """
        Public method to hide the item.
        """
        self.setVisible(False)
        
    def show(self):
        """
        Public method to show the item.
        """
        self.setVisible(True)
        
    def setSelected(self, selected):
        """
        Public method to set the items selection status.
        
        @param selected selection status (boolean)
        """
        self._selected = selected
        
    def isSelected(self):
        """
        Public method to check the items selection status.
        
        @return flag indicating the selection status (boolean)
        """
        return self._selected
        
    def move(self, x, y):
        """
        Public method to move the item to an absolute position.
        
        @param x new x-coordinate (float)
        @param y new y-coordinate (float)
        """
        self.setX(x)
        self.setY(y)
        
        origin = self.getOrigin()
        size = self._boundingRect.size()
        self._boundingRect.setCoords(origin.x(), origin.y(), 
            origin.x() + size.width(), origin.y() + size.height())
        self._path = QPainterPath()
        
        self._adaptCanvasSize()
        self._canvas.update()
        
    def moveBy(self, dx, dy):
        """
        Public method to move the item relative to its current position.
        
        @param dx relative x movement (float)
        @param dy relative y movement (float)
        """
        self.move(self._x + dx, self._y + dy)
        
    def drawSelected(self, painter):#, x, y):
        """
        Public method to draw the widget as selected.
        
        @param painter painter to draw on (QPainter)
        @param x x-position (integer)
        @param y y-position (integer)
        """
        x = self._boundingRect.x()
        y = self._boundingRect.y()
        w = self._boundingRect.width()
        h = self._boundingRect.height()
        s = 6
        brush = QBrush(Qt.red)
        painter.fillRect(x, y, s, s, brush)
        painter.fillRect(x, y + h - s, s, s, brush)
        painter.fillRect(x + w - s, y, s, s, brush)
        painter.fillRect(x + w - s, y + h - s, s, s, brush)
    
    ############################################################################
    ## Bounding rectangle management
    ############################################################################
    
    def boundingRect(self):
        """
        Public method to get the bounding rectangle.
        
        @return reference to the bounding rectangle (QRectF)
        """
        return self._boundingRect
    
    ############################################################################
    ## Protected methods
    ############################################################################
    
    def _adaptCanvasSize(self):
        """
        Protected method to update the size of the canvas.
        
        This method does never shrink the canvas.
        """
        canvasSize = self._canvas.getCanvasSize()
        if self._boundingRect.right() > canvasSize.width() or \
           self._boundingRect.bottom() > canvasSize.height():
            w = max(self._boundingRect.right(), canvasSize.width())
            h = max(self._boundingRect.bottom(), canvasSize.height())
            self._canvas.setCanvasSize(w + 10, h + 10)
        
    def update(self):
        """
        Protected method to update the canvas' changed parts.
        """
        self._canvas.update()
    
    ############################################################################
    ## The following method must be overridden by subclasses
    ############################################################################
    
    def drawShape(self, painter):
        """
        Public method to draw the shape.
        
        <b>Note</b>: This method must be overridden by implementation classes.
        
        @param painter reference to the painter object (QPainter)
        """
        raise RuntimeError("Not implemented")
