# -*- coding: utf-8 -*-

# Copyright (c) 2006 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a canvas item subclass for a line.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from E4CanvasItem import E4CanvasItem

class E4CanvasLine(E4CanvasItem):
    """
    Class implementing a line canvas item subclass.
    """
    def __init__(self, canvas, origin = QPointF(0.0, 0.0), end = QPointF(0.0, 0.0)):
        """
        Constructor
        
        @param canvas reference to the canvas object (E4Canvas)
        @param origin origin of the line (QPointF)
        @param end end point of the line (QPointF)
        """
        E4CanvasItem.__init__(self, canvas, origin)
        
        self._end = end
        
    def setPoints(self, xa, ya, xb, yb):
        """
        Public method to set the start and end points of the line.
        
        @param xa x-coordinate of the start point (float)
        @param ya y-coordinate of the start point (float)
        @param xb x-coordinate of the end point (float)
        @param yb y-coordinate of the end point (float)
        """
        self.setX(xa)
        self.setY(ya)
        self._end.setX(xb)
        self._end.setY(yb)
        self._path = QPainterPath()
        
    def setStartPoint(self, x, y):
        """
        Public method to set the start point.
        
        @param x x-coordinate of the start point (float)
        @param y y-coordinate of the start point (float)
        """
        self.setX(x)
        self.setY(y)
        self._path = QPainterPath()
        
    def setEndPoint(self, x, y):
        """
        Public method to set the end point.
        
        @param x x-coordinate of the end point (float)
        @param y y-coordinate of the end point (float)
        """
        self._end.setX(x)
        self._end.setY(y)
        self._path = QPainterPath()
        
    def startPoint(self):
        """
        Public method to get the start point of the line.
        
        @return start point (QPointF)
        """
        return self.getOrigin()
        
    def endPoint(self):
        """
        Public method to get the end point of the line.
        
        @return end point (QPointF)
        """
        return self._end
        
    def move(self, x, y):
        """
        Public method to move the item to an absolute position.
        
        @param x new x-coordinate (float)
        @param y new y-coordinate (float)
        """
        dx = x - self.x()
        dy = y - self.y()
        self.setX(x)
        self.setY(y)
        self._end.setX(self._end.x() + dx)
        self._end.setY(self._end.y() + dy)
        
        origin = self.getOrigin()
        size = self._boundingRect.size()
        self._boundingRect.setCoords(origin.x(), origin.y(), 
            origin.x() + size.width(), origin.y() + size.height())
        self._path = QPainterPath()
        
        self._adaptCanvasSize()
        self._canvas.update()
        
    def moveBy(self, dx, dy):
        """
        Public method to move the item relative to its current position.
        
        @param dx relative x movement (float)
        @param dy relative y movement (float)
        """
        self.move(self._x + dx, self._y + dy)
        
    def drawShape(self, painter):
        """
        Public method to draw the shape.
        
        @param painter reference to the painter object (QPainter)
        """
        if not self.isVisible():
            return
        
        painter.setPen(self.getPen())
        
        if self._path.isEmpty():
            self._path.moveTo(self.x(), self.y())
            self._path.lineTo(self._end.x(), self._end.y())
            self._boundingRect = self._path.boundingRect()
        
        painter.drawPath(self._path)
        if self.isSelected():
            self.drawSelected(painter)
