# -*- coding: utf-8 -*-

# Copyright (c) 2006 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a canvas view class.
"""

import sys

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from E4CanvasRectangle import E4CanvasRectangle

class E4CanvasView(QScrollArea):
    """
    Class implementing a canvas view.
    """
    def __init__(self, parent, canvas = None):
        """
        Constructor
        
        @param parent parent widget (QWidget)
        @param canvas reference to the canvas to be shown (E4Canvas)
        """
        QScrollArea.__init__(self, parent)
        self.setObjectName("E4CanvasView")
        
        self._canvas = canvas
        if self._canvas:
            self.setWidget(self._canvas)
        
        palette = self.viewport().palette()
        palette.setColor(self.backgroundRole(), palette.color(QPalette.Dark))
        self.viewport().setPalette(palette)
        
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        
        self.setWhatsThis(self.trUtf8("<b>Canvas View</b>\n"
            "<p>This canvas view is used to show a diagram. \n"
            "There are various actions available to manipulate the \n"
            "shown items.</p>\n"
            "<ul>\n"
            "<li>Clicking on an item selects it.</li>\n"
            "<li>Shift-clicking adds an item to the selection.</li>\n"
            "<li>Shift-clicking a selected item deselects it.</li>\n"
            "<li>Clicking on an empty spot of the canvas resets the selection.</li>\n"
            "<li>Dragging the mouse over the canvas spans a rubberband to \n"
            "select multiple items.</li>\n"
            "<li>Shift-dragging the mouse over the canvas spans a rubberband \n"
            "to add multiple items to the selection</li>\n"
            "<li>Dragging the mouse over a selected item moves the \n"
            "whole selection.</li>\n"
            "</ul>\n"
        ))
        
        self.clear()
        
        self.selectionRect = QRect(0, 0, 1, 1)
        self.oldPosX = 0
        self.oldPosY = 0
        
        # set this in a subclass to have a border around items when moving
        self.border = 0
        
        self.dragging = False
        self.rubberbanding = False
        self.rubberband = None
        if self._canvas:
            self.rubberband = E4CanvasRectangle(self._canvas, QPointF(5, 5))
            self.rubberband.getPen().setStyle(Qt.DotLine)
            self.rubberband.hide()
        
        self.update()
        
    def setCanvas(self, canvas):
        """
        Public method to set the canvas object.
        
        @param reference to the canvas object (E4Canvas)
        """
        if self._canvas:
            self.takeWidget(self._canvas)
        self._canvas = canvas
        self.setWidget(self._canvas)
        self.rubberbanding = False
        self.rubberband = E4CanvasRectangle(self.canvas(), QPointF(5, 5))
        self.rubberband.getPen().setStyle(Qt.DotLine)
        self.rubberband.hide()
        self.dragging = False
        
    def canvas(self):
        """
        Public method to get a reference to the canvas object.
        
        @return canvas object (E4Canvas)
        """
        return self._canvas
        
    def clear(self):
        """
        Public method to clear the canvas view.
        """
        self._canvas.clear()
        
    def zoomIn(self):
        """
        Public method to zoom in.
        """
        self._canvas.setZoom(self._canvas.zoom() * 1.25)
        
    def zoomOut(self):
        """
        Public method to zoom out.
        """
        self._canvas.setZoom(self._canvas.zoom() * 0.8)
    
    def zoomReset(self):
        """
        Public method to handle the reset zoom context menu entry.
        """
        self._canvas.setZoom(1.0)
        
    def setZoom(self, zoomFactor):
        """
        Public method to set the zoom factor.
        
        @param zoomFactor new zoom factor (float)
        """
        self._canvas.setZoom(zoomFactor)
        
    def zoom(self):
        """
        Public method to get the current zoom factor.
        
        @return current zoom factor (float)
        """
        return self._canvas.zoom()
        
    def mousePressEvent(self, evt):
        """
        Overriden method to handle mouse button presses.
        
        This method determines the item over which the button
        press occurred.
        
        @param evt mouse event (QMouseEvent)
        """
        # we're only interested in pure and shifted button presses
        if evt.modifiers() & Qt.KeyboardModifiers(Qt.ControlModifier | Qt.AltModifier):
            evt.ignore()
            return
        if evt.button() == Qt.LeftButton:
            pos = self.canvas().mapFromParent(evt.pos())
            pos /= self.zoom()
            if pos.x() > self.canvas().getCanvasWidth() or \
               pos.y() > self.canvas().getCanvasHeight():
                return
            l = self.collisions(self.canvas().collisions(pos))
            if len(l):
                if len(l) == 1:
                    if evt.modifiers() & Qt.KeyboardModifiers(Qt.ShiftModifier):
                        self.selectItem(l[0])
                    else:
                        if not l[0].isSelected():
                            self.selectItems(l)
                
                srl = self.canvas().width()
                srr = 0
                srt = self.canvas().height()
                srb = 0
                for item in self.canvas().getSelectedItems():
                    ix = int(item.x())
                    iy = int(item.y())
                    iw = item.boundingRect().width()
                    ih = item.boundingRect().height()
                    srl = min(ix, srl)
                    srt = min(iy, srt)
                    srr = max(ix + iw, srr)
                    srb = max(iy + ih, srb)
                self.selectionRect.setRect(srl, srt, srr - srl, srb - srt)
                self.oldPosX = pos.x()
                self.oldPosY = pos.y()
            else:
                if not (evt.modifiers() & Qt.KeyboardModifiers(Qt.ShiftModifier)):
                    self.unselectItems()
                    self.selectionRect.setRect(0, 0, 1, 1)
                if not self.rubberbanding:
                    # start rubberbanding
                    self.rubberbanding = True
                    self.rubberband.setSize(0, 0)
                    self.rubberband.move(pos.x(), pos.y())
                    self.rubberband.show()
        
        self.canvas().update()
        
    def mouseReleaseEvent(self, evt):
        """
        Overriden method to handle mouse button releases.
        
        This method simply resets the tracked widget.
        
        @param evt mouse event (QMouseEvent)
        """
        if evt.button() == Qt.LeftButton:
            if self.rubberbanding:
                self.rubberbanding = False
                collisionItems = self.collisions(\
                        self.canvas().collisions(self.rubberband.boundingRect()))
                for item in collisionItems:
                    item.setSelected(True)
                self.rubberband.hide()
                self.rubberband.setSize(0, 0)
                self.rubberband.move(5, 5)
                self.canvas().update()
                return
        
    def mouseMoveEvent(self, evt):
        """
        Overriden method to handle mouse moves.
        
        This method moves the widget according to the mouse
        movements.
        
        @param evt mouse event (QMouseEvent)
        """
        pos = self.canvas().mapFromParent(evt.pos())
        spos = pos / self.zoom()
        if spos.x() > self.canvas().getCanvasWidth():
            spos.setX(self.canvas().getCanvasWidth())
        if spos.y() > self.canvas().getCanvasHeight():
            spos.setY(self.canvas().getCanvasHeight())
        if self.rubberbanding:
            self.rubberband.setSize(int(spos.x() - self.rubberband.x()),
                                    int(spos.y() - self.rubberband.y()))
            self.canvas().update()
            self.ensureVisible(pos.x(), pos.y(), 10, 10)
        elif self.canvas().getSelectedItems():
            self.dragging = True
            cw = self.canvas().getCanvasWidth() - self.border
            ch = self.canvas().getCanvasHeight() - self.border
            newX = spos.x()
            newY = spos.y()
            if newX < self.border:
                newX = self.border
            elif newX > cw:
                newX = cw
            if newY < self.border:
                newY = self.border
            elif newY > ch:
                newY = ch
            offsetX = newX - self.oldPosX
            offsetY = newY - self.oldPosY
            self.oldPosX = newX
            self.oldPosY = newY
            srl = self.selectionRect.left()
            srr = self.selectionRect.right()
            srt = self.selectionRect.top()
            srb = self.selectionRect.bottom()
            if srl + offsetX < self.border and offsetX < 0:
                offsetX = srl - self.border
            elif srr + offsetX > cw and offsetX > 0:
                offsetX = cw - srr
            if srt + offsetY < self.border and offsetY < 0:
                offsetY = srt - self.border
            elif srb + offsetY > ch and offsetY > 0:
                offsetY = ch - srb
            self.selectionRect.moveTo(self.selectionRect.x() + offsetX, 
                                      self.selectionRect.y() + offsetY)
            for item in self.canvas().getSelectedItems():
                item.moveBy(offsetX, offsetY)
            self.canvas().update()
            self.ensureVisible(pos.x(), pos.y(), 10, 10)
        
    def resizeCanvas(self, amount, isWidth = True):
        """
        Public method to resize the drawing canvas.
        
        @param isWidth flag indicating width is to be resized (boolean)
        @param amount size increment (integer)
        """
        width = self.canvas().getCanvasWidth()
        height = self.canvas().getCanvasHeight()
        if isWidth:
            width += amount
        else:
            height += amount
        rect = self._getDiagramRect(10)
        if width < rect.width():
            width = rect.width()
        if height < rect.height():
            height = rect.height()
        self.setCanvasSize(width, height)
        
    def setCanvasSize(self, width, height):
        """
        Public method to set the canvas size.
        
        @param width width for the canvas (integer)
        @param height height for the canvas (integer)
        """
        self.canvas().setCanvasSize(width, height)
        
    def saveImage(self, filename, format = "PNG"):
        """
        Public method to save the canvas to a file.
        
        @param filename name of the file to write the image to (string or QString)
        @param format for the image file (string or QString)
        @return flag indicating success (boolean)
        """
        rect = self._getDiagramRect(self.border)
        pixmap = self.__getDiagram(rect)
        return pixmap.save(filename, format)
        
    def _getDiagramRect(self, border = 0):
        """
        Protected method to calculate the minimum rectangle fitting the diagram.
        
        @param border border width to include in the calculation (integer)
        @return the minimum rectangle (QRect)
        """
        startx = sys.maxint
        starty = sys.maxint
        endx = 0
        endy = 0
        items = self.collisions(self.canvas().getItems())
        for itm in items:
            rect = itm.boundingRect()
            itmEndX = rect.x() + rect.width()
            itmEndY = rect.y()+ rect.height()
            itmStartX = rect.x()
            itmStartY = rect.y()
            if startx >= itmStartX:
                startx = itmStartX
            if starty >= itmStartY:
                starty = itmStartY
            if endx <= itmEndX:
                endx = itmEndX
            if endy <= itmEndY:
                endy = itmEndY
        if border:
            startx -= border
            starty -= border
            endx += border
            endy += border
            
        return QRect(int(startx), int(starty), 
                     int(endx - startx + 1), int(endy - starty + 1))
        
    def __getDiagram(self, rect):
        """
        Private method to retrieve the diagram from the canvas fitting it 
        in the minimum rectangle.
        
        @param rect minimum rectangle fitting the diagram (QRect)
        @return diagram pixmap to receive the diagram (QPixmap)
        """
        selectedItems = self.canvas().getSelectedItems()
        
        # step 1: deselect all widgets
        if selectedItems:
            for item in selectedItems:
                item.setSelected(False)
            self.canvas().update()
            
        # step 2: grab the diagram
        pixmap = QPixmap.grabWidget(self.canvas(), rect)
        
        # step 3: reselect the widgets
        if selectedItems:
            for item in selectedItems:
                item.setSelected(True)
            self.canvas().update()
        
        return pixmap
        
    def printDiagram(self, printer, diagramName = ""):
        """
        Public method to print the diagram.
        
        @param printer reference to a ready configured printer object (QPrinter)
        @param diagramName name of the diagram (string or QString)
        """
        painter = QPainter()
        painter.begin(printer)
        offsetX = 0
        offsetY = 0
        widthX = 0
        heightY = 0
        font = QFont("times", 10)
        painter.setFont(font)
        fm = painter.fontMetrics()
        fontHeight = fm.lineSpacing()
        marginX = printer.pageRect().x() - printer.paperRect().x()
        marginY = printer.pageRect().y() - printer.paperRect().y()
        
##        # double the margin on bottom of page
##        if printer.orientation() == QPrinter.Portrait:
##            width = printer.width() - marginX * 2
##            height = printer.height() - fontHeight - 4 - marginY * 3
##        else:
##            marginX *= 2
##            width = printer.width() - marginX * 2
##            height = printer.height() - fontHeight - 4 - marginY * 2
        width = printer.width() - marginX * 2
        height = printer.height() - fontHeight - 4 - marginY * 2
        
        border = self.border == 0 and 5 or self.border
        rect = self._getDiagramRect(border)
        diagram = self.__getDiagram(rect)
        
        finishX = False
        finishY = False
        page = 0
        pageX = 0
        pageY = 1
        while not finishX or not finishY:
            if not finishX:
                offsetX = pageX * width
                pageX += 1
            elif not finishY:
                offsetY = pageY * height
                offsetX = 0
                pageY += 1
                finishX = False
                pageX = 1
            if (width + offsetX) > diagram.width():
                finishX = True
                widthX = diagram.width() - offsetX
            else:
                widthX = width
            if diagram.width() < width:
                widthX = diagram.width()
                finishX = True
                offsetX = 0
            if (height + offsetY) > diagram.height():
                finishY = True
                heightY = diagram.height() - offsetY
            else:
                heightY = height
            if diagram.height() < height:
                finishY = True
                heightY = diagram.height()
                offsetY = 0
            
            painter.drawPixmap(marginX, marginY, diagram,
                               offsetX, offsetY, widthX, heightY)
            # write a foot note
            s = QString(self.trUtf8("Diagram: %1, Page %2")
                .arg(diagramName).arg(page + 1))
            tc = QColor(50, 50, 50)
            painter.setPen(tc)
            painter.drawRect(marginX, marginY, width, height)
            painter.drawLine(marginX, marginY + height + 2, 
                       marginX + width, marginY + height + 2)
            painter.setFont(font)
            painter.drawText(marginX, marginY + height + 4, width,
                       fontHeight, Qt.AlignRight, s)
            if not finishX or not finishY:
                printer.newPage()
                page += 1
        
        painter.end()
    
    ############################################################################
    ## The methods below should be overridden by subclasses to get special
    ## behavior.
    ############################################################################
    
    def collisions(self, items):
        """
        Public method to filter a list of collisions.
        
        @param items list of collision items as reported by the canvas object
            (E4CanvasItem)
        @return list of interesting collision items (E4CanvasItem)
        """
        # just return the list unchanged
        try:
            items.remove(self.rubberband)
        except:
            pass
        return items
        
    def unselectItems(self):
        """
        Public method to unselect all items.
        """
        for itm in self.canvas().getSelectedItems():
            itm.setSelected(False)
        
    def selectItems(self, items):
        """
        Public method to select the given items.
        
        @param items list of items to be selected (list of E4CanvasItem)
        """
        # step 1: deselect all items
        self.unselectItems()
        
        # step 2: select all given items
        for itm in items:
            itm.setSelected(True)
        
    def selectItem(self, item):
        """
        Public method to select an item.
        
        @param item item to be selected (E4CanvasItem)
        """
        item.setSelected(not item.isSelected())
