# -*- coding: utf-8 -*-

# Copyright (c) 2004 - 2006 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a subclass of QCanvasView for our diagrams.
"""

import sys

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from E4Canvas.E4CanvasView import E4CanvasView

from KdeQt import KQFileDialog, KQMessageBox
import KdeQt.KQPrinter
from KdeQt.KQPrintDialog import KQPrintDialog

from UMLWidget import UMLWidget
from UMLCanvasSizeDialog import UMLCanvasSizeDialog
from ZoomDialog import ZoomDialog

import Preferences

class UMLCanvasView(E4CanvasView):
    """
    Class implementing a specialized canvasview for our diagrams.
    """
    def __init__(self, canvas, parent = None, name = None):
        """
        Constructor
        
        @param canvas canvas that is managed by the view (QCanvas)
        @param parent parent widget of the view (QWidget)
        @param name name of the view widget (QString or string)
        """
        E4CanvasView.__init__(self, parent, canvas)
        if name:
            self.setObjectName(name)
        
        self.border = 10
        self.deltaSize = 100.0
        
        self.menuActions = {}
        self.__menu = QMenu(self)
        self.menuActions["DelShapes"] = \
            self.__menu.addAction(self.trUtf8("Delete shapes"), self.__deleteShape)
        self.__menu.addSeparator()
        self.__menu.addAction(self.trUtf8("Save as PNG"), self.__saveImage)
        self.__menu.addAction(self.trUtf8("Print"), self.__printDiagram)
        self.__menu.addSeparator()
        self.__menu.addAction(self.trUtf8("Zoom in"), self.zoomIn)
        self.__menu.addAction(self.trUtf8("Zoom out"), self.zoomOut)
        self.__menu.addAction(self.trUtf8("Zoom..."), self.__zoom)
        self.__menu.addAction(self.trUtf8("Zoom reset"), self.zoomReset)
        self.__menu.addSeparator()
        self.__menu.addAction(self.trUtf8("Increase width by %1 points")\
            .arg(self.deltaSize), self.__incWidth)
        self.__menu.addAction(self.trUtf8("Increase height by %1 points")\
            .arg(self.deltaSize), self.__incHeight)
        self.menuActions["DecWidth"] = \
            self.__menu.addAction(self.trUtf8("Decrease width by %1 points")\
            .arg(self.deltaSize), self.__decWidth)
        self.menuActions["DecHeight"] = \
            self.__menu.addAction(self.trUtf8("Decrease height by %1 points")\
            .arg(self.deltaSize), self.__decHeight)
        self.__menu.addAction(self.trUtf8("Set size"), self.__setSize)
        self.__menu.addSeparator()
        self.__menu.addAction(self.trUtf8("Re-Layout"), self.__relayout)
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.connect(self, SIGNAL('customContextMenuRequested(const QPoint &)'),
                     self.__showContextMenu)
        
    def __showContextMenu(self, coord):
        """
        Private slot to show the context menu.
        
        @param coord the position of the mouse pointer (QPoint)
        """
        rect = self._getDiagramRect(10)
        if self.canvas().getCanvasWidth() <= rect.width():
            self.menuActions["DecWidth"].setEnabled(False)
        else:
            self.menuActions["DecWidth"].setEnabled(True)
        if self.canvas().getCanvasHeight() <= rect.height():
            self.menuActions["DecHeight"].setEnabled(False)
        else:
            self.menuActions["DecHeight"].setEnabled(True)
        if len(self.canvas().getSelectedItems()) > 0:
            self.menuActions["DelShapes"].setEnabled(True)
        else:
            self.menuActions["DelShapes"].setEnabled(False)
        self.__menu.popup(self.mapToGlobal(coord))
        
    def collisions(self, items):
        """
        Public method to filter a list of collisions.
        
        @param items list of collision items as reported by the canvas object
            (E4CanvasItem)
        @return list of interesting collision items (E4CanvasItem)
        """
        return [itm for itm in items if isinstance(itm, UMLWidget)]
        
    def selectItems(self, items):
        """
        Public method to select the given items.
        
        @param items list of items to be selected (list of E4CanvasItem)
        """
        # step 1: deselect all items
        self.unselectItems()
        
        # step 2: select all given items
        for itm in items:
            if isinstance(itm, UMLWidget):
                itm.setSelected(True)
        
    def selectItem(self, item):
        """
        Public method to select an item.
        
        @param item item to be selected (E4CanvasItem)
        """
        if isinstance(item, UMLWidget):
            item.setSelected(not item.isSelected())
        
    def __deleteShape(self):
        """
        Private method to delete the selected shapes from the display.
        """
        for widget in self.canvas().getSelectedItems():
            widget.removeAssociations()
            widget.setSelected(False)
            widget.hide()
            del widget
        self.canvas().update()
        
    def __incWidth(self):
        """
        Private method to handle the increase width context menu entry.
        """
        self.resizeCanvas(self.deltaSize, True)
        
    def __incHeight(self):
        """
        Private method to handle the increase height context menu entry.
        """
        self.resizeCanvas(self.deltaSize, False)
        
    def __decWidth(self):
        """
        Private method to handle the decrease width context menu entry.
        """
        self.resizeCanvas(-self.deltaSize, True)
        
    def __decHeight(self):
        """
        Private method to handle the decrease height context menu entry.
        """
        self.resizeCanvas(-self.deltaSize, False)
        
    def __setSize(self):
        """
        Private method to handle the set size context menu entry.
        """
        cv = self.canvas()
        rect = self._getDiagramRect(10)
        dlg = UMLCanvasSizeDialog(cv.getCanvasWidth(), cv.getCanvasHeight(),
                                  rect.width(), rect.height(), self)
        if dlg.exec_() == QDialog.Accepted:
            width, height = dlg.getData()
            self.setCanvasSize(width, height)
        
    def __saveImage(self):
        """
        Private method to handle the save context menu entry.
        """
        selectedFilter = QString('')
        fname = KQFileDialog.getSaveFileName(\
            self,
            self.trUtf8("Save Diagram"),
            QString(),
            self.trUtf8("Portable Network Graphics (*.png)"),
            selectedFilter,
            QFileDialog.Options(QFileDialog.DontConfirmOverwrite))
        if not fname.isEmpty():
            ext = QFileInfo(fname).suffix()
            if ext.isEmpty():
                ex = selectedFilter.section('(*',1,1).section(')',0,0)
                if not ex.isEmpty():
                    fname.append(ex)
            if QFileInfo(fname).exists():
                res = KQMessageBox.warning(self,
                    self.trUtf8("Save Diagram"),
                    self.trUtf8("<p>The file <b>%1</b> already exists.</p>")
                        .arg(fname),
                    QMessageBox.StandardButtons(\
                        QMessageBox.Abort | \
                        QMessageBox.Save),
                    QMessageBox.Abort)
                if res == QMessageBox.Abort or res == QMessageBox.Cancel:
                    return
            
            success = self.saveImage(fname, "PNG")
            if not success:
                KQMessageBox.critical(None,
                    self.trUtf8("Save Diagram"),
                    self.trUtf8("""<p>The file <b>%1</b> could not be saved.</p>""")
                        .arg(fname))
        
    def __relayout(self):
        """
        Private method to handle the re-layout context menu entry.
        """
        cv = self.canvas()
        for itm in cv.getItems()[:]:
            cv.removeItem(itm)
        self.parent().relayout()
        cv.update()
        
    def __printDiagram(self):
        """
        Private slot called to print the diagram.
        """
        printer = KdeQt.KQPrinter.KQPrinter()
        printer.setFullPage(True)
        if Preferences.getPrinter("ColorMode"):
            printer.setColorMode(KdeQt.KQPrinter.Color)
        else:
            printer.setColorMode(KdeQt.KQPrinter.GrayScale)
        if Preferences.getPrinter("FirstPageFirst"):
            printer.setPageOrder(KdeQt.KQPrinter.FirstPageFirst)
        else:
            printer.setPageOrder(KdeQt.KQPrinter.LastPageFirst)
        printer.setPrinterName(Preferences.getPrinter("PrinterName"))
        
        printDialog = KQPrintDialog(printer, self)
        if printDialog.exec_():
            self.printDiagram(printer, self.parent().getDiagramName())
        
    def __zoom(self):
        """
        Private method to handle the zoom context menu action.
        """
        dlg = ZoomDialog(self.zoom(), self)
        if dlg.exec_() == QDialog.Accepted:
            zoom = dlg.getZoomSize()
            self.setZoom(zoom)
