# -*- coding: utf-8 -*-

# Copyright (c) 2004 - 2006 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a dialog showing a UML like class diagram.
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from Utilities.ModuleParser import readModule

from E4Canvas.E4Canvas import E4Canvas
from E4Canvas.E4CanvasText import E4CanvasText

from UMLCanvasView import UMLCanvasView
from ClassWidget import ClassWidget, ClassModel
from AssociationWidget import AssociationWidget, Generalisation
import GraphicsUtilities

_allClasses = {}
_allModules = {}

class UMLClassDiagram(QDialog):
    """
    Class implementing a dialog showing a UML like class diagram.
    """
    def __init__(self, file, parent = None, name = None, noAttrs = False):
        """
        Constructor
        
        @param file filename of a python module to be shown (string)
        @param parent parent widget of the view (QWidget)
        @param name name of the view widget (QString or string)
        @keyparam noAttrs flag indicating, that no attributes should be shown (boolean)
        """
        QDialog.__init__(self, parent)
        
        if not name:
            self.setObjectName("UMLClassDiagram")
        else:
            self.setObjectName(name)
        
        self.UMLFormLayout = QVBoxLayout(self)
        self.UMLFormLayout.setMargin(6)
        self.UMLFormLayout.setSpacing(6)
        self.UMLFormLayout.setObjectName("UMLFormLayout")
        self.setLayout(self.UMLFormLayout)
        
        self.canvas = E4Canvas(None, 800, 600)
        self.umlCanvas = UMLCanvasView(self.canvas, self, "umlCanvas")
        self.UMLFormLayout.addWidget(self.umlCanvas)
        
        # generate the buttons
        buttonLayout = QHBoxLayout()
        buttonLayout.setMargin(0)
        buttonLayout.setSpacing(6)
        buttonLayout.setObjectName("layout1")
        
        leftSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        buttonLayout.addItem(leftSpacer)
        
        self.closeButton = QPushButton(self)
        self.closeButton.setObjectName("closeButton")
        self.closeButton.setDefault(True)
        self.closeButton.setText(self.trUtf8("&Close"))
        self.closeButton.setShortcut(self.trUtf8("Alt+C"))
        buttonLayout.addWidget(self.closeButton)
        self.connect(self.closeButton, SIGNAL("clicked()"), self, SLOT("close()"))
        
        rightSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        buttonLayout.addItem(rightSpacer)
        
        self.UMLFormLayout.addLayout(buttonLayout)
        
        self.file = file
        self.noAttrs = noAttrs
        
    def getDiagramName(self):
        """
        Method to retrieve a name for the diagram.
        
        @return name for the diagram
        """
        return self.file
        
    def __getCurrentShape(self, name):
        """
        Private method to get the named shape.
        
        @param name name of the shape (string)
        @return shape (E4CanvasItem)
        """
        return _allClasses.get(name)
        
    def __buildClasses(self):
        """
        Private method to build the class shapes of the class diagram.
        
        The algorithm is borrowed from Boa Constructor.
        """
        try:
            module = readModule(self.file)
        except ImportError:
            ct = E4CanvasText(self.canvas, QPointF(10.0, 10.0),
                              self.trUtf8("The module '%1' could not be found.")
                                .arg(self.file))
            ct.show()
            return
        
        if not _allModules.has_key(self.file):
            _allModules[self.file] = []
        
        routes = []
        nodes = []
        todo = [module.createHierarchy()]
        classesFound = False
        while todo:
            hierarchy = todo[0]
            for className in hierarchy.keys():
                classesFound = True
                cw = self.__getCurrentShape(className)
                if not cw and className.find('.') >= 0:
                    cw = self.__getCurrentShape(className.split('.')[-1])
                    if cw:
                        _allClasses[className] = cw
                        if className not in _allModules[self.file]:
                            _allModules[self.file].append(className)
                if cw and cw.noAttrs != self.noAttrs:
                    cw = None
                if cw and not (cw.external and \
                               (module.classes.has_key(className) or
                                module.modules.has_key(className))
                              ):
                    cw.setCanvas(self.canvas)
                    cw.move(10, 10)
                    cw.show()
                    if className not in nodes:
                        nodes.append(className)
                else:
                    if module.classes.has_key(className):
                        # this is a local class (defined in this module)
                        self.__addLocalClass(\
                            className, module.classes[className], 10, 10)
                    elif module.modules.has_key(className):
                        # this is a local module (defined in this module)
                        self.__addLocalClass(\
                            className, module.modules[className], 10, 10, True)
                    else:
                        self.__addExternalClass(className, 10, 10)
                    nodes.append(className)
                
                if hierarchy.get(className):
                    todo.append(hierarchy.get(className))
                    children = hierarchy.get(className).keys()
                    for child in children:
                        if (className, child) not in routes:
                            routes.append((className, child))
            
            del todo[0]
        
        if classesFound:
            self.__arrangeClasses(nodes, routes[:])
            self.__createAssociations(routes)
        else:
            ct = E4CanvasText(self.canvas, QPointF(10.0, 10.0),
                              self.trUtf8("The module '%1' does not contain any classes.")
                                .arg(self.file))
            ct.show()
        
    def __arrangeClasses(self, nodes, routes, whiteSpaceFactor = 1.2):
        """
        Private method to arrange the shapes on the canvas.
        
        The algorithm is borrowed from Boa Constructor.
        """
        generations = GraphicsUtilities.sort(nodes, routes)
        
        # calculate width and height of all elements
        sizes = []
        for generation in generations:
            sizes.append([])
            for child in generation:
                sizes[-1].append(self.__getCurrentShape(child).boundingRect())
        
        # calculate total width and total height
        width = 0
        height = 0
        widths = []
        heights = []
        for generation in sizes:
            currentWidth = 0
            currentHeight = 0
            
            for rect in generation:
                if rect.bottom() > currentHeight:
                    currentHeight = rect.bottom()
                currentWidth = currentWidth + rect.right()
            
            # update totals
            if currentWidth > width:
                width = currentWidth
            height = height + currentHeight
            
            # store generation info
            widths.append(currentWidth)
            heights.append(currentHeight)
        
        # add in some whitespace
        width = width * whiteSpaceFactor
        rawHeight = height
        height = height * whiteSpaceFactor - 20
        verticalWhiteSpace = (height - rawHeight) / (len(generations) - 1.0 or 2.0)
        
        width += 50.0
        height += 50.0
        cwidth = width < self.canvas.getCanvasWidth() and \
            self.canvas.getCanvasWidth() or width
        cheight = height < self.canvas.getCanvasHeight() and \
            self.canvas.getCanvasHeight() or height
        self.canvas.setCanvasSize(cwidth, cheight)
        
        # distribute each generation across the width and the
        # generations across height
        y = 10.0
        for currentWidth, currentHeight, generation in \
                map(None, widths, heights, generations):
            x = 10.0
            # whiteSpace is the space between any two elements
            whiteSpace = (width - currentWidth - 20) / (len(generation) - 1.0 or 2.0)
            for className in generation:
                cw = self.__getCurrentShape(className)
                cw.move(x, y)
                rect = cw.boundingRect()
                x = x + rect.width() + whiteSpace
            y = y + currentHeight + verticalWhiteSpace
        
    def __addLocalClass(self, className, _class, x, y, isRbModule = False):
        """
        Private method to add a class defined in the module.
        
        If the canvas is too small to take the shape, it
        is enlarged.
        
        @param className name of the class to be as a dictionary key (string)
        @param _class class to be shown (ModuleParser.Class)
        @param x x-coordinate (float)
        @param y y-coordinate (float)
        @param isRbModule flag indicating a Ruby module (boolean)
        """
        meths = _class.methods.keys()
        meths.sort()
        attrs = _class.attributes[:]
        attrs.sort()
        name = _class.name
        if isRbModule:
            name = "%s (Module)" % name
        cl = ClassModel(name, meths[:], attrs[:])
        cw = ClassWidget(self.canvas, cl, False, x, y, noAttrs = self.noAttrs)
        cw.show()
        _allClasses[className] = cw
        if _class.name not in _allModules[self.file]:
            _allModules[self.file].append(_class.name)
        
    def __addExternalClass(self, _class, x, y):
        """
        Private method to add a class defined outside the module.
        
        If the canvas is too small to take the shape, it
        is enlarged.
        
        @param _class class to be shown (string)
        @param x x-coordinate (float)
        @param y y-coordinate (float)
        """
        cl = ClassModel(_class)
        cw = ClassWidget(self.canvas, cl, True, x, y)
        cw.show()
        _allClasses[_class] = cw
        if _class not in _allModules[self.file]:
            _allModules[self.file].append(_class)
        
    def __createAssociations(self, routes):
        """
        Private method to generate the associations between the class shapes.
        
        @param routes list of relationsships
        """
        for route in routes:
            if len(route) > 1:
                assoc = AssociationWidget(self.canvas, 
                        self.__getCurrentShape(route[1]),
                        self.__getCurrentShape(route[0]),
                        Generalisation)
                assoc.show()
        
    def show(self):
        """
        Overriden method to show the dialog.
        """
        self.__buildClasses()
        QDialog.show(self)
        
    def relayout(self):
        """
        Public method to relayout the diagram.
        """
        _allClasses.clear()
        _allModules.clear()
        self.__buildClasses()

def resetCachedWidgets():
    """
    Module function to reset the list of cached widgets.
    """
    _allClasses.clear()
    _allModules.clear()
    
def resetCachedWidgetsByFile(filename):
    """
    Module function to reset the list of cached widgets belonging to a file.
    
    @param file filename of the widgets to be removed from cache (string)
    """
    if _allModules.has_key(filename):
        for name in _allModules[filename]:
            if _allClasses.has_key(name):
                del _allClasses[name]
        del _allModules[filename]
