# -*- coding: utf-8 -*-

# Copyright (c) 2004 - 2006 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a dialog to compare two files and show the result side by side.
"""

import os
import sys
from difflib import SequenceMatcher

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from KdeQt import KQFileDialog, KQMessageBox

from Ui_CompareDialog import Ui_CompareDialog
import Utilities

def sbsdiff(a, b, linenumberwidth = 4):
    """
    Compare two sequences of lines; generate the delta for display side by side.
    
    @param a first sequence of lines (list of strings)
    @param b second sequence of lines (list of strings)
    @param linenumberwidth width (in characters) of the linenumbers (integer)
    @return a generator yielding tuples of differences. The tuple is composed
        of strings as follows.
        <ul>
            <li>opcode -- one of e, d, i, r for equal, delete, insert, replace</li>
            <li>lineno a -- linenumber of sequence a</li>
            <li>line a -- line of sequence a</li>
            <li>lineno b -- linenumber of sequence b</li>
            <li>line b -- line of sequence b</li>
        </ul>
    """
    linenumberformat = "%%%dd" % linenumberwidth
    emptylineno = ' ' * linenumberwidth
    
    for tag, i1, i2, j1, j2 in SequenceMatcher(None,a,b).get_opcodes():
        if tag == 'equal':
            for x in range(i2 - i1):
                yield ('e', str(linenumberformat % (i1+x+1)), a[i1+x],
                            str(linenumberformat % (j1+x+1)), b[j1+x])
            continue
        if tag == 'delete':
            for x in range(i2 - i1):
                yield ('d', str(linenumberformat % (i1+x+1)),
                            a[i1+x], emptylineno, '\n')
            continue
        if tag == 'insert':
            for x in range(j2 - j1):
                yield ('i', emptylineno, '\n',
                            str(linenumberformat % (j1+x+1)), b[j1+x])
            continue
        if tag == 'replace':
            for x in range(max(i2-i1, j2-j1)):
                if i1 + x < i2:
                    s1 = str(linenumberformat % (i1+x+1))
                    s2 = a[i1+x]
                else:
                    s1 = emptylineno
                    s2 = '\n'
                if j1 + x < j2:
                    s3 = str(linenumberformat % (j1+x+1))
                    s4 = b[j1+x]
                else:
                    s3 = emptylineno
                    s4 = '\n'
                yield ('r', s1, s2, s3, s4)

class CompareDialog(QWidget, Ui_CompareDialog):
    """
    Class implementing a dialog to compare two files and show the result side by side.
    """
    def __init__(self,parent = None):
        """
        Constructor
        
        @param parent parent widget (QWidget)
        """
        QWidget.__init__(self,parent)
        self.setupUi(self)
        
        self.updateInterval = 20    # update every 20 lines
        
        self.vsb1 = self.contents_1.verticalScrollBar()
        self.hsb1 = self.contents_1.horizontalScrollBar()
        self.vsb2 = self.contents_2.verticalScrollBar()
        self.hsb2 = self.contents_2.horizontalScrollBar()
        
        self.on_synchronizeCheckBox_toggled(True)
        
        if sys.platform == "win32":
            self.contents_1.setFontFamily("Lucida Console")
            self.contents_2.setFontFamily("Lucida Console")
        else:
            self.contents_1.setFontFamily("Monospace")
            self.contents_2.setFontFamily("Monospace")
        
        self.cNormalFormat = self.contents_1.currentCharFormat()
        self.cInsertedFormat = self.contents_1.currentCharFormat()
        self.cInsertedFormat.setBackground(QBrush(QColor(190, 237, 190)))
        self.cDeletedFormat = self.contents_1.currentCharFormat()
        self.cDeletedFormat.setBackground(QBrush(QColor(237, 190, 190)))
        self.cReplacedFormat = self.contents_1.currentCharFormat()
        self.cReplacedFormat.setBackground(QBrush(QColor(190, 190, 237)))
        
        # connect some of our widgets explicitly
        self.connect(self.file1Edit, SIGNAL("textChanged(const QString &)"), 
                     self.__fileChanged)
        self.connect(self.file2Edit, SIGNAL("textChanged(const QString &)"), 
                     self.__fileChanged)
        self.connect(self.synchronizeCheckBox, SIGNAL("toggled(bool)"), 
                     self.on_synchronizeCheckBox_toggled)

    def __appendText(self, pane, txt, format):
        """
        Private method to append text to the end of the contents pane.
        
        @param pane text edit widget to append text to (QTextedit)
        @param txt text to insert (QString)
        @param format text format to be used (QTextCharFormat)
        """
        tc = pane.textCursor()
        tc.movePosition(QTextCursor.End)
        pane.setTextCursor(tc)
        pane.setCurrentCharFormat(format)
        pane.insertPlainText(txt)
        
    @pyqtSignature("")
    def on_diffButton_clicked(self):
        """
        Private slot to handle the Compare button press.
        """
        filename1 = unicode(Utilities.toNativeSeparators(self.file1Edit.text()))
        try:
            f1 = open(filename1, "rb")
            lines1 = f1.readlines()
            f1.close()
        except IOError:
            KQMessageBox.critical(self,
                self.trUtf8("Compare Files"),
                self.trUtf8("""<p>The file <b>%1</b> could not be read.</p>""")
                    .arg(filename1))
            return

        filename2 = unicode(Utilities.toNativeSeparators(self.file2Edit.text()))
        try:
            f2 = open(filename2, "rb")
            lines2 = f2.readlines()
            f2.close()
        except IOError:
            KQMessageBox.critical(self,
                self.trUtf8("Compare Files"),
                self.trUtf8("""<p>The file <b>%1</b> could not be read.</p>""")
                    .arg(filename2))
            return
        
        self.contents_1.clear()
        self.contents_2.clear()
        
        paras = 1
        for opcode, ln1, l1, ln2, l2 in sbsdiff(lines1, lines2):
            p1 = "%s %s" % (ln1, l1)
            p2 = "%s %s" % (ln2, l2)
            if opcode == 'i':
                format1 = self.cNormalFormat
                format2 = self.cInsertedFormat
            elif opcode == 'd':
                format1 = self.cDeletedFormat
                format2 = self.cNormalFormat
            elif opcode == 'r':
                if ln1.strip():
                    format1 = self.cReplacedFormat
                else:
                    format1 = self.cNormalFormat
                if ln2.strip():
                    format2 = self.cReplacedFormat
                else:
                    format2 = self.cNormalFormat
            else:
                format1 = self.cNormalFormat
                format2 = self.cNormalFormat
            self.__appendText(self.contents_1, p1, format1)
            self.__appendText(self.contents_2, p2, format2)
            paras += 1
            if not (paras % self.updateInterval):
                QApplication.processEvents()
        
        tc = self.contents_1.textCursor()
        tc.movePosition(QTextCursor.Start)
        self.contents_1.setTextCursor(tc)
        self.contents_1.ensureCursorVisible()
        tc = self.contents_2.textCursor()
        tc.movePosition(QTextCursor.Start)
        self.contents_2.setTextCursor(tc)
        self.contents_2.ensureCursorVisible()

    def __fileChanged(self):
        """
        Private slot to enable/disable the Compare button.
        """
        if self.file1Edit.text().isEmpty() or \
           self.file2Edit.text().isEmpty():
            self.diffButton.setEnabled(False)
        else:
            self.diffButton.setEnabled(True)

    def __selectFile(self, lineEdit):
        """
        Private slot to display a file selection dialog.
        
        @param lineEdit field for the display of the selected filename
                (QLineEdit)
        """
        filename = KQFileDialog.getOpenFileName(\
            self,
            self.trUtf8("Select file to compare"),
            lineEdit.text(),
            QString())
            
        if not filename.isEmpty():
            lineEdit.setText(Utilities.toNativeSeparators(filename))

    @pyqtSignature("")
    def on_file1Button_clicked(self):
        """
        Private slot to handle the file 1 file selection button press.
        """
        self.__selectFile(self.file1Edit)

    @pyqtSignature("")
    def on_file2Button_clicked(self):
        """
        Private slot to handle the file 2 file selection button press.
        """
        self.__selectFile(self.file2Edit)

    @pyqtSignature("bool")
    def on_synchronizeCheckBox_toggled(self, sync):
        """
        Private slot to connect or disconnect the scrollbars of the displays.
        
        @param sync flag indicating synchronisation status (boolean)
        """
        if sync:
            self.vsb2.setValue(self.vsb1.value())
            self.hsb2.setValue(self.hsb1.value())
            self.connect(self.vsb1, SIGNAL('valueChanged(int)'),
                self.vsb2, SLOT('setValue(int)'))
            self.connect(self.vsb2, SIGNAL('valueChanged(int)'),
                self.vsb1, SLOT('setValue(int)'))
            self.connect(self.hsb1, SIGNAL('valueChanged(int)'),
                self.hsb2, SLOT('setValue(int)'))
            self.connect(self.hsb2, SIGNAL('valueChanged(int)'),
                self.hsb1, SLOT('setValue(int)'))
        else:
            self.disconnect(self.vsb1, SIGNAL('valueChanged(int)'),
                self.vsb2, SLOT('setValue(int)'))
            self.disconnect(self.vsb2, SIGNAL('valueChanged(int)'),
                self.vsb1, SLOT('setValue(int)'))
            self.disconnect(self.hsb1, SIGNAL('valueChanged(int)'),
                self.hsb2, SLOT('setValue(int)'))
            self.disconnect(self.hsb2, SIGNAL('valueChanged(int)'),
                self.hsb1, SLOT('setValue(int)'))
